@@ -19,10 +19,13 @@ module Hledger.Web.Handler.MiscR
   , getManageR
   , getRobotsR
   , getRootR
+  , getOpenApiR
   ) where
 
 import qualified Data.Map as M
 import qualified Data.Text as T
+import qualified Data.Yaml as Yaml
+import qualified Data.ByteString as BS
 import Yesod.Default.Handlers (getFaviconR, getRobotsR)
 
 import Hledger
@@ -84,7 +87,7 @@ getCommoditiesR = do
   VD{j} <- getViewData
   require ViewPermission
   selectRep $ do
-    provideJson $ (M.keys . jinferredcommodities) j
+    provideJson $ (M.keys . jinferredcommoditystyles) j
 
 getAccountsR :: Handler TypedContent
 getAccountsR = do
@@ -107,3 +110,13 @@ getAccounttransactionsR a = do
       styleAmounts (journalCommodityStylesWith HardRounding j) $
       accountTransactionsReport rspec{_rsQuery=Any} j thisacctq
 
+openApiYaml :: BS.ByteString
+openApiYaml = $(embedFileRelative "config/openapi.yaml")
+
+getOpenApiR :: Handler Value
+getOpenApiR =
+  case Yaml.decodeEither' openApiYaml of
+    Left _ -> notFound
+    Right openapi -> do
+      addHeader "Content-Type" "application/json"
+      return openapi
