@@ -11,6 +11,7 @@ module Hledger.Web.Handler.RegisterR where
 
 import Data.List (intersperse, nub, partition)
 import qualified Data.Text as T
+import Safe (tailSafe)
 import Text.Hamlet (hamletFile)
 
 import Hledger
@@ -44,7 +45,7 @@ getRegisterR = do
       addCommas xs =
           zip xs $
           zip (map (T.unpack . accountSummarisedName . paccount) xs) $
-          tail $ (", "<$xs) ++ [""]
+          tailSafe (", "<$xs) ++ [""]
       items =
         styleAmounts (journalCommodityStylesWith HardRounding j) $
         accountTransactionsReport rspec{_rsQuery=q} j acctQuery
@@ -92,7 +93,7 @@ undecorateLinks xs0@(x:_) =
             let (link, xs1) = span (isJust . fst) xs0
                 (comma, xs2) = span (isNothing . fst) xs1
             in (acct, (map snd link, map snd comma)) : undecorateLinks xs2
-        _ -> error "link name not decorated with account"  -- PARTIAL:
+        _ -> error' "link name not decorated with account"  -- PARTIAL:
 
 decorateLinks :: [(acct, ([char], [char]))] -> [(Maybe acct, char)]
 decorateLinks = concatMap $ \(acct, (name, comma)) ->
@@ -108,14 +109,14 @@ registerChartHtml q title percommoditytxnreports = $(hamletFile "templates/chart
    charttitle = if null title then "" else title ++ ":"
    colorForCommodity = fromMaybe 0 . flip lookup commoditiesIndex
    commoditiesIndex = zip (map fst percommoditytxnreports) [0..] :: [(CommoditySymbol,Int)]
-   simpleMixedAmountQuantity = maybe 0 aquantity . listToMaybe . amounts . mixedAmountStripPrices
-   showZeroCommodity = wbUnpack . showMixedAmountB oneLine{displayCost=False,displayZeroCommodity=True}
+   simpleMixedAmountQuantity = maybe 0 aquantity . listToMaybe . amounts . mixedAmountStripCosts
+   showZeroCommodity = wbUnpack . showMixedAmountB oneLineNoCostFmt{displayCost=False,displayZeroCommodity=True}
    shownull c = if null c then " " else c
    nodatelink = (RegisterR, [("q", T.unwords $ removeDates q)])
 
-dayToJsTimestamp :: Day -> Integer
-dayToJsTimestamp d =
+-- | Makes a unix timestamp (milliseconds since epoch) corresponding to noon on the given date in UTC.
+dayToUtcNoonTimestamp :: Day -> Integer
+dayToUtcNoonTimestamp d =
   read (formatTime defaultTimeLocale "%s" t) * 1000 -- XXX read
   where
-    t = UTCTime d (secondsToDiffTime 0)
-
+    t = UTCTime d (secondsToDiffTime $ 12 * 60 * 60)
