@@ -1,10 +1,23 @@
 {-|
 hledger-web - a basic but robust web UI and JSON API server for hledger.
 
-Copyright (c) 2007-2023 Simon Michael <simon@joyful.com> and contributors.
-Released under GPL version 3 or later.
+SPDX-License-Identifier: GPL-3.0-or-later
+Copyright (c) 2007-2025 (each year in this range) Simon Michael <simon@joyful.com> and contributors.
+
+This program is free software: you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+See the GNU General Public License for more details.
+You should have received a copy of the GNU General Public License along with this program.
+If not, see <https://www.gnu.org/licenses/>.
+
 -}
 
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE MultiWayIf #-}
 {-# LANGUAGE NamedFieldPuns #-}
 {-# LANGUAGE OverloadedStrings #-}
@@ -12,6 +25,9 @@ Released under GPL version 3 or later.
 module Hledger.Web.Main where
 
 import Control.Exception (bracket)
+#if MIN_VERSION_base(4,20,0)
+import Control.Exception.Backtrace (setBacktraceMechanismState, BacktraceMechanism(..))
+#endif
 import Control.Monad (when)
 import Data.String (fromString)
 import qualified Data.Text as T
@@ -21,7 +37,6 @@ import Network.Wai.Handler.Warp (runSettings, runSettingsSocket, defaultSettings
 import Network.Wai.Handler.Launch (runHostPortFullUrl)
 import System.Directory (removeFile)
 import System.Environment ( getArgs, withArgs )
-import System.Exit (exitSuccess, exitFailure)
 import System.IO (hFlush, stdout)
 import System.PosixCompat.Files (getFileStatus, isSocket)
 import Text.Printf (printf)
@@ -38,7 +53,7 @@ import Hledger.Web.WebOptions
 -- Run in fast reloading mode for yesod devel.
 hledgerWebDev :: IO (Int, Application)
 hledgerWebDev =
-  withJournalDo (cliopts_ defwebopts) (defaultDevelApp loader . makeApplication defwebopts)
+  withJournal (cliopts_ defwebopts) (defaultDevelApp loader . makeApplication defwebopts)
   where
     loader =
       Yesod.Default.Config.loadConfig
@@ -46,22 +61,41 @@ hledgerWebDev =
 
 -- Run normally.
 hledgerWebMain :: IO ()
-hledgerWebMain = do
+hledgerWebMain = handleExit $ withGhcDebug' $ do
+  when (ghcDebugMode == GDPauseAtStart) $ ghcDebugPause'
+
+#if MIN_VERSION_base(4,20,0)
+  -- Control ghc 9.10+'s stack traces.
+  -- CostCentreBacktrace   - collect cost-centre stack backtraces (only available when built with profiling)
+  -- HasCallStackBacktrace - collect HasCallStack backtraces
+  -- ExecutionBacktrace    - collect backtraces from native execution stack unwinding
+  -- IPEBacktrace          - collect backtraces from Info Table Provenance Entries
+#ifdef DEBUG
+  setBacktraceMechanismState HasCallStackBacktrace True
+#else
+  setBacktraceMechanismState HasCallStackBacktrace False
+#endif
+#endif
+
   -- try to encourage user's $PAGER to properly display ANSI (in command line help)
-  when useColorOnStdout setupPager
+  usecolor <- useColorOnStdout
+  when usecolor setupPager
 
   wopts@WebOpts{cliopts_=copts@CliOpts{debug_, rawopts_}} <- getHledgerWebOpts
   when (debug_ > 0) $ printf "%s\n" prognameandversion >> printf "opts: %s\n" (show wopts)
   if
-    | boolopt "help"            rawopts_ -> pager (showModeUsage webmode) >> exitSuccess
+    | boolopt "help"            rawopts_ -> runPager $ showModeUsage webmode ++ "\n"
+    | boolopt "tldr"            rawopts_ -> runTldrForPage "hledger-web"
     | boolopt "info"            rawopts_ -> runInfoForTopic "hledger-web" Nothing
     | boolopt "man"             rawopts_ -> runManForTopic  "hledger-web" Nothing
-    | boolopt "version"         rawopts_ -> putStrLn prognameandversion >> exitSuccess
+    | boolopt "version"         rawopts_ -> putStrLn prognameandversion
     -- boolopt "binary-filename" rawopts_ -> putStrLn (binaryfilename progname)
     | boolopt "test"            rawopts_ -> do
       -- remove --test and --, leaving other args for hspec
       (`withArgs` hledgerWebTest) . filter (`notElem` ["--test","--"]) =<< getArgs
-    | otherwise                              -> withJournalDo copts (web wopts)
+    | otherwise                              -> withJournal copts (web wopts)
+
+  when (ghcDebugMode == GDPauseAtEnd) $ ghcDebugPause'
 
 -- | The hledger web command.
 web :: WebOpts -> Journal -> IO ()
@@ -82,8 +116,9 @@ web opts j = do
 
   -- show configuration
   let
-    services | serve_api_ opts = "json API"
-             | otherwise       = "web UI and json API"
+    services
+      | server_mode_ opts == ServeJson = "json API"
+      | otherwise                      = "web UI and json API"
     prettyip ip
         | ip == "127.0.0.1" = ip ++ " (local access)"
         | ip == "0.0.0.0"   = ip ++ " (all interfaces)"
@@ -98,8 +133,15 @@ web opts j = do
     Nothing -> pure ()
 
   -- start server and maybe browser
-  if serve_ opts || serve_api_ opts
+  if server_mode_ opts == ServeBrowse
     then do
+      putStrLn "This server will exit after 2m with no browser windows open (or press ctrl-c)"
+      putStrLn "Opening web browser..."
+      hFlush stdout
+      -- exits after 2m of inactivity (hardcoded)
+      Network.Wai.Handler.Launch.runHostPortFullUrl h p u app
+
+    else do
       putStrLn "Press ctrl-c to quit"
       hFlush stdout
       let warpsettings = setHost (fromString h) (setPort p defaultSettings)
@@ -119,17 +161,10 @@ web opts j = do
                   when (isSocket sockstat) $ removeFile s
               )
               (\sock -> Network.Wai.Handler.Warp.runSettingsSocket warpsettings sock app)
-            else do
-              putStrLn "Unix domain sockets are not available on your operating system"
-              putStrLn "Please try again without --socket"
-              exitFailure
+            else error' $ unlines
+              ["Unix domain sockets are not available on your operating system."
+              ,"Please try again without --socket."
+              ]
 
         Nothing -> Network.Wai.Handler.Warp.runSettings warpsettings app
 
-    else do
-      putStrLn "This server will exit after 2m with no browser windows open (or press ctrl-c)"
-      putStrLn "Opening web browser..."
-      hFlush stdout
-      -- exits after 2m of inactivity (hardcoded)
-      Network.Wai.Handler.Launch.runHostPortFullUrl h p u app
-
