@@ -1,5 +1,8 @@
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE RecordWildCards #-}
+{-# LANGUAGE LambdaCase #-}
 
 module Hledger.Web.WebOptions where
 
@@ -9,6 +12,7 @@ import Data.ByteString.UTF8 (fromString)
 import Data.Default (Default(def))
 import Data.Maybe (fromMaybe)
 import Data.Text (Text)
+import GitHash (tGitInfoCwdTry)
 import System.Environment (getArgs)
 import Network.Wai as WAI
 import Network.Wai.Middleware.Cors
@@ -18,10 +22,12 @@ import Hledger.Cli hiding (packageversion, progname, prognameandversion)
 import Hledger.Web.Settings (defhost, defport, defbaseurl)
 import qualified Data.Text as T
 import Data.Char (toLower)
+import Data.List (isPrefixOf)
+import Data.Either (fromRight)
 
 -- cf Hledger.Cli.Version
 
-packageversion :: PackageVersion
+packageversion :: PackageVersionString
 packageversion =
 #ifdef VERSION
   VERSION
@@ -29,23 +35,41 @@ packageversion =
   ""
 #endif
 
+-- | The name of this program's executable.
 progname :: ProgramName
 progname = "hledger-web"
 
-prognameandversion :: VersionString
-prognameandversion = versionString progname packageversion
+-- | Generate the version string for this program.
+-- The template haskell call is here rather than in Hledger.Cli.Version to avoid wasteful recompilation.
+prognameandversion :: String
+prognameandversion =
+  versionStringWith
+  $$tGitInfoCwdTry
+#ifdef GHCDEBUG
+  True
+#else
+  False
+#endif
+  progname
+  packageversion
 
+binaryinfo :: HledgerBinaryInfo
+binaryinfo = fromRight nullbinaryinfo $ parseHledgerVersion prognameandversion
 
 webflags :: [Flag RawOpts]
 webflags =
   [ flagNone
-      ["serve", "server"]
+      ["serve-browse"]
+      (setboolopt "serve-browse")
+      (serveprefix ++ "serve the web UI and JSON API, and open a browser, and exit if inactive for 2m (default)")
+  , flagNone
+      ["serve"]
       (setboolopt "serve")
-      "serve and log requests, don't browse or auto-exit"
+      (serveprefix ++ "just serve the web UI and JSON API")
   , flagNone
       ["serve-api"]
       (setboolopt "serve-api")
-      "like --serve, but serve only the JSON web API, not the web UI"
+      (serveprefix ++ "just serve the JSON API")
   , flagReq
       ["allow"]
       (\s opts -> Right $ setopt "allow" s opts)
@@ -70,7 +94,7 @@ webflags =
       ["socket"]
       (\s opts -> Right $ setopt "socket" s opts)
       "SOCKET"
-      "listen on the given unix socket instead of an IP address and port (unix only; implies --serve)"
+      "listen on the given unix socket instead of an IP address and port (only on unix)"
   , flagReq
       ["base-url"]
       (\s opts -> Right $ setopt "base-url" s opts)
@@ -87,6 +111,8 @@ webflags =
       (setboolopt "test")
       "run hledger-web's tests and exit. hspec test runner args may follow a --, eg: hledger-web --test -- --help"
   ]
+  where
+    serveprefix = "server mode: "
 
 webmode :: Mode RawOpts
 webmode =
@@ -99,23 +125,19 @@ webmode =
   { modeGroupFlags =
       Group
       { groupUnnamed = webflags
-      , groupHidden =
-          hiddenflags 
-          -- ++
-          -- [ flagNone
-          --     ["binary-filename"]
-          --     (setboolopt "binary-filename")
-          --     "show the download filename for this executable, and exit"
-          -- ]
-      , groupNamed = [generalflagsgroup1]
+      , groupHidden = hiddenflags
+          ++
+          [flagNone ["server"] (setboolopt "serve") "old flag, use --serve instead"]
+      , groupNamed = mkgeneralflagsgroups1 helpflags
       }
   , modeHelpSuffix = []
   }
 
+data ServerMode = ServeBrowse | Serve | ServeJson deriving (Show, Eq)
+
 -- hledger-web options, used in hledger-web and above
 data WebOpts = WebOpts
-  { serve_              :: !Bool
-  , serve_api_          :: !Bool
+  { server_mode_        :: !ServerMode
   , cors_               :: !(Maybe String)
   , host_               :: !String
   , port_               :: !Int
@@ -128,8 +150,7 @@ data WebOpts = WebOpts
 
 defwebopts :: WebOpts
 defwebopts = WebOpts
-  { serve_              = False
-  , serve_api_          = False
+  { server_mode_        = ServeBrowse
   , cors_               = Nothing
   , host_               = ""
   , port_               = def
@@ -146,23 +167,25 @@ rawOptsToWebOpts :: RawOpts -> IO WebOpts
 rawOptsToWebOpts rawopts =
   checkWebOpts <$> do
     cliopts <- rawOptsToCliOpts rawopts
-    let h = fromMaybe defhost $ maybestringopt "host" rawopts
-        p = fromMaybe defport $ maybeposintopt "port" rawopts
-        b = maybe (defbaseurl h p) stripTrailingSlash $ maybestringopt "base-url" rawopts
-        sock = stripTrailingSlash <$> maybestringopt "socket" rawopts
-        access =
-          case lastMay $ listofstringopt "allow" rawopts of
-            Nothing -> AddAccess
-            Just t ->
-              case parseAccessLevel t of
-                Right al -> al
-                Left err -> error' ("Unknown access level: " ++ err)  -- PARTIAL:
+    let
+      h = fromMaybe defhost $ maybestringopt "host" rawopts
+      p = fromMaybe defport $ maybeposintopt "port" rawopts
+      -- Always set a base-url, constructing it from host and port if not specified.
+      -- This will be used when opening a web browser, eg.
+      -- App.hs approot will use it if it was specified by --base-url,
+      -- otherwise it will infer a better one from the request, which browsers prefer.
+      b = maybe (defbaseurl h p) stripTrailingSlash $ maybestringopt "base-url" rawopts
+      sock = stripTrailingSlash <$> maybestringopt "socket" rawopts
+      access =
+        case lastMay $ listofstringopt "allow" rawopts of
+          Nothing -> AddAccess
+          Just t ->
+            case parseAccessLevel t of
+              Right al -> al
+              Left err -> error' ("Unknown access level: " ++ err)  -- PARTIAL:
     return
       defwebopts
-      { serve_ = case sock of
-          Just _ -> True
-          Nothing -> boolopt "serve" rawopts
-      , serve_api_ = boolopt "serve-api" rawopts
+      { server_mode_ = servermodeopt rawopts
       , cors_ = maybestringopt "cors" rawopts
       , host_ = h
       , port_ = p
@@ -175,12 +198,25 @@ rawOptsToWebOpts rawopts =
   where
     stripTrailingSlash = reverse . dropWhile (== '/') . reverse -- yesod don't like it
 
+servermodeopt :: RawOpts -> ServerMode
+servermodeopt =
+  fromMaybe ServeBrowse . choiceopt parse
+ where
+  parse = \case
+    "serve-browse" -> Just ServeBrowse
+    "serve"        -> Just Serve
+    "serve-api"    -> Just ServeJson
+    _ -> Nothing
+
 checkWebOpts :: WebOpts -> WebOpts
-checkWebOpts = id
+checkWebOpts wopts@WebOpts{..}
+  | not $ null base_url_ || "http://" `isPrefixOf` base_url_ || "https://" `isPrefixOf` base_url_ =
+      error' "please begin the --base-url value with http:// or https://"
+  | otherwise = wopts
 
 getHledgerWebOpts :: IO WebOpts
 getHledgerWebOpts = do
-  args <- fmap (replaceNumericFlags . ensureDebugHasArg) . expandArgsAt =<< getArgs
+  args <- fmap (ensureDebugFlagHasVal . replaceNumericFlags) . expandArgsAt =<< getArgs
   rawOptsToWebOpts . either usageError id $ process webmode args
 
 data Permission
